# -*- coding: utf-8 -*-
import asyncio
import sys

import aioaria2


async def main():
    server = aioaria2.AsyncAria2Server(
        r"128aria2c.exe", r"--conf-path=aria2.conf", "--rpc-secret=admin", daemon=True
    )
    await server.start()
    await server.wait()
    print("不等他")


if __name__ == "__main__":
    # if sys.platform == "win32":
    #     # 想不到吧，官方文档说要在windows使用asyncio.create_subprocess_shell，只说了修改event_loop_policy，
    #     # 没想到还要修改默认的事件循环，哈哈
    #     # windows就是矫情
    #     asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
    #     asyncio.set_event_loop(asyncio.ProactorEventLoop())
    # asyncio.run(main())
    # main()
    def a(c=1, d=2):
        print(c, d)

    a(None)



# -*- coding: utf-8 -*-
"""
支持类型注释
"""
from typing import TYPE_CHECKING, Any, Awaitable, Callable, Dict, TypeVar, Union

Aria2WebsocketClient = TypeVar("Aria2WebsocketClient", bound="Aria2WebsocketClient")

if TYPE_CHECKING:
    from aioaria2 import Aria2WebsocketClient

"""
Websocket事件的回调函数
"""
CallBack = Callable[
    [Aria2WebsocketClient, Dict[str, Any]], Union[Awaitable[Any], Awaitable[None]]
]

"""
产生随机id的工厂函数 如果一定要参数可以用functools.partial
"""
IdFactory = Callable[[], Union[int, Awaitable[int]]]



import asyncio
import unittest
from pprint import pprint

import ujson

import aioaria2


class TestHTTP(unittest.IsolatedAsyncioTestCase):
    async def asyncSetUp(self) -> None:
        self.client = aioaria2.Aria2HttpClient(
            "http://aria.blackjoe.art:2082/jsonrpc", token="a489451594cda0792df1"
        )

    async def test_addUri(self):
        gid = await self.client.addUri(["https://www.google.com"])
        self.assertTrue(isinstance(gid, str))

    async def test_getVersion(self):
        data = await self.client.getVersion()
        self.assertTrue(isinstance(data, dict))

    async def asyncTearDown(self) -> None:
        await self.client.close()


class TestWebsocket(unittest.IsolatedAsyncioTestCase):
    onDownloadStart1 = None
    onDownloadStart2 = None
    onDownloadComplete1 = None
    onDownloadComplete2 = None

    async def asyncSetUp(self) -> None:
        # self.client = aioaria2.Aria2HttpClient("test", "http://aria.blackjoe.art:2082/jsonrpc",
        #                                        token="a489451594cda0792df1")
        self.trigger = await aioaria2.Aria2WebsocketTrigger.new(
            "http://aria.blackjoe.art:2082/jsonrpc",
            token="a489451594cda0792df1",
            loads=ujson.loads,
            dumps=ujson.dumps,
        )

        # asyncio.get_running_loop().create_task(self.trigger.listen())
        @self.trigger.onDownloadStart
        async def handeler1(trigger, data):
            print("我是1号回调,我收到了消息")
            self.__class__.onDownloadStart1 = data

        @self.trigger.onDownloadStart
        @aioaria2.run_sync
        def handeler2(trigger, data):
            print("我是2号回调,我收到了消息")
            self.__class__.onDownloadStart2 = data

        @self.trigger.onDownloadComplete
        async def handeler3(trigger, data):
            print("我是3号回调,我收到了消息")
            self.__class__.onDownloadComplete1 = data

        @self.trigger.onDownloadComplete
        @aioaria2.run_sync
        def handeler4(trigger, data):
            print("我是4号回调,我收到了消息")
            self.__class__.onDownloadComplete2 = data

        await asyncio.sleep(10)

    async def test_connect(self):
        data = await self.trigger.getVersion()
        pprint(data)
        print(self.trigger.functions)

        self.assertTrue(isinstance(data, dict))
        self.assertTrue(
            len(self.trigger.functions["aria2.onDownloadStart"]) == 2,
            len(self.trigger.functions["aria2.onDownloadStart"]),
        )
        self.assertTrue(
            len(self.trigger.functions["aria2.onDownloadComplete"]) == 2,
            len(self.trigger.functions["aria2.onDownloadComplete"]),
        )

    async def test_onDownloadStart_and_Stop(self):
        data = await self.trigger.addUri(["https://www.baidu.com"])
        pprint(data)
        self.assertTrue(isinstance(data, str), data)
        while True:
            if self.onDownloadComplete1 is not None:
                self.assertEqual(
                    self.onDownloadStart1["method"],
                    "aria2.onDownloadStart",
                    "回调断言失败,期待{0} 接收到了{1}".format(
                        "aria2.onDownloadStart", self.onDownloadStart1["method"]
                    ),
                )
                self.assertEqual(
                    self.onDownloadStart2["method"],
                    "aria2.onDownloadStart",
                    "回调断言失败,期待{0} 接收到了{1}".format(
                        "aria2.onDownloadStart", self.onDownloadStart1["method"]
                    ),
                )
                break
            await asyncio.sleep(1)
        while True:
            if self.onDownloadComplete1 is not None:
                self.assertEqual(
                    self.onDownloadComplete1["method"],
                    "aria2.onDownloadComplete",
                    "回调断言失败,期待{0} 接收到了{1}".format(
                        "aria2.onDownloadComplete", self.onDownloadComplete1["method"]
                    ),
                )
                self.assertEqual(
                    self.onDownloadComplete2["method"],
                    "aria2.onDownloadComplete",
                    "回调断言失败,期待{0} 接收到了{1}".format(
                        "aria2.onDownloadComplete", self.onDownloadComplete2["method"]
                    ),
                )
                break
            await asyncio.sleep(1)

    async def asyncTearDown(self) -> None:
        await self.trigger.close()


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
See https://aria2.github.io/manual/en/html/technical-notes.html
"""
from dataclasses import dataclass
from ipaddress import IPv4Address, IPv6Address
from pathlib import Path
from typing import IO, List, Union


@dataclass
class InFlightPiece:
    index: int
    length: int
    piece_bitfield_length: int
    piece_bitfield: bytes

    @classmethod
    def from_file(cls, file: IO[bytes], version: int) -> "InFlightPiece":
        index = int.from_bytes(file.read(4), "big" if version == 1 else "little")
        length = int.from_bytes(file.read(4), "big" if version == 1 else "little")
        piece_bitfield_length = int.from_bytes(
            file.read(4), "big" if version == 1 else "little"
        )
        piece_bitfield = file.read(piece_bitfield_length)
        return cls(
            index=index,
            length=length,
            piece_bitfield_length=piece_bitfield_length,
            piece_bitfield=piece_bitfield,
        )

    def save(self, file: IO[bytes], version: int) -> None:
        file.write(self.index.to_bytes(4, "big" if version == 1 else "little"))
        file.write(self.length.to_bytes(4, "big" if version == 1 else "little"))
        file.write(
            len(self.piece_bitfield).to_bytes(4, "big" if version == 1 else "little")
        )
        file.write(self.piece_bitfield)


@dataclass
class ControlFile:
    """
    Parse .aria2 files
    """

    version: int
    ext: bytes
    info_hash_length: int
    info_hash: bytes
    piece_length: int
    total_length: int
    upload_length: int
    bitfield_length: int
    bitfield: bytes
    num_inflight_piece: int
    inflight_pieces: List[InFlightPiece]

    @classmethod
    def from_file(cls, file: Union[str, Path, IO[bytes]]) -> "ControlFile":
        should_close: bool = False
        try:
            if isinstance(file, (str, Path)):
                file_ = open(file, "rb")
                should_close = True
            else:
                file_ = file  # type: ignore
            version = int.from_bytes(file_.read(2), "big")
            ext = file_.read(4)
            info_hash_length = int.from_bytes(
                file_.read(4), "big" if version == 1 else "little"
            )
            if info_hash_length == 0 and ext[3] & 1 == 1:
                raise ValueError(
                    '"infoHashCheck" extension is enabled but info hash length is 0'
                )
            info_hash = file_.read(info_hash_length)
            piece_length = int.from_bytes(
                file_.read(4), "big" if version == 1 else "little"
            )
            total_length = int.from_bytes(
                file_.read(8), "big" if version == 1 else "little"
            )
            upload_length = int.from_bytes(
                file_.read(8), "big" if version == 1 else "little"
            )
            bitfield_length = int.from_bytes(
                file_.read(4), "big" if version == 1 else "little"
            )
            bitfield = file_.read(bitfield_length)
            num_inflight_piece = int.from_bytes(
                file_.read(4), "big" if version == 1 else "little"
            )
            inflight_pieces = [
                InFlightPiece.from_file(file_, version)
                for _ in range(num_inflight_piece)
            ]

            return cls(
                version=version,
                ext=ext,
                info_hash_length=info_hash_length,
                info_hash=info_hash,
                piece_length=piece_length,
                total_length=total_length,
                upload_length=upload_length,
                bitfield_length=bitfield_length,
                bitfield=bitfield,
                num_inflight_piece=num_inflight_piece,
                inflight_pieces=inflight_pieces,
            )
        finally:
            if should_close:
                try:
                    file_.close()
                except:
                    pass

    def save(self, file: IO[bytes]) -> None:
        file.write(self.version.to_bytes(2, "big" if self.version == 1 else "little"))
        file.write(self.ext)
        file.write(
            len(self.info_hash).to_bytes(4, "big" if self.version == 1 else "little")
        )
        file.write(self.info_hash)
        file.write(
            self.piece_length.to_bytes(4, "big" if self.version == 1 else "little")
        )
        file.write(
            self.total_length.to_bytes(8, "big" if self.version == 1 else "little")
        )
        file.write(
            self.upload_length.to_bytes(8, "big" if self.version == 1 else "little")
        )
        file.write(
            len(self.bitfield).to_bytes(4, "big" if self.version == 1 else "little")
        )
        file.write(self.bitfield)
        file.write(
            len(self.inflight_pieces).to_bytes(
                4, "big" if self.version == 1 else "little"
            )
        )
        for piece in self.inflight_pieces:
            piece.save(file, self.version)


@dataclass
class NodeInfo:
    plen: int
    compact_peer_info: tuple
    node_id: bytes

    @classmethod
    def from_file(cls, file: IO[bytes]) -> "NodeInfo":
        plen = int.from_bytes(file.read(1), "big")
        file.read(7)
        class_ = IPv4Address if plen == 6 else IPv6Address
        temp = file.read(plen)
        compact_peer_info = (class_(temp[:-2]), int.from_bytes(temp[-2:], "big"))
        file.read(24 - plen)
        node_id = file.read(20)
        file.read(4)
        return cls(plen=plen, compact_peer_info=compact_peer_info, node_id=node_id)

    def save(self, file: IO[bytes]) -> None:
        file.write(self.plen.to_bytes(1, "big"))
        file.write(b"\x00" * 7)

        file.write(self.compact_peer_info[0].packed)
        file.write(self.compact_peer_info[1].to_bytes(2, "big"))

        file.write(b"\x00" * (24 - self.plen))
        file.write(self.node_id)
        file.write(b"\x00" * 4)


@dataclass
class DHTFile:
    """
    Parse dht.dat/dht6.dat files
    """

    mgc: bytes
    fmt: bytes
    ver: bytes
    mtime: int
    localnode_id: bytes
    num_node: int
    nodes: List[NodeInfo]

    @classmethod
    def from_file(cls, file: Union[str, Path, IO[bytes]]) -> "DHTFile":
        should_close: bool = False
        try:
            if isinstance(file, (str, Path)):
                file_ = open(file, "rb")
                should_close = True
            else:
                file_ = file  # type: ignore
            mgc = file_.read(2)
            assert mgc == b"\xa1\xa2", "wrong magic number"
            fmt = file_.read(1)
            assert fmt == b"\x02", "wrong format idr"
            ver = file_.read(2)
            # assert ver == b'\x00\x03', "wrong version number"
            file_.read(3)
            mtime = int.from_bytes(file_.read(8), "big")
            file_.read(8)
            localnode_id = file_.read(20)
            file_.read(4)
            num_node = int.from_bytes(file_.read(4), "big")
            file_.read(4)
            nodes = [NodeInfo.from_file(file_) for _ in range(num_node)]
            return cls(
                mgc=mgc,
                fmt=fmt,
                ver=ver,
                mtime=mtime,
                localnode_id=localnode_id,
                num_node=num_node,
                nodes=nodes,
            )
        finally:
            if should_close:
                try:
                    file_.close()
                except:
                    pass

    def save(self, file: IO[bytes]) -> None:
        file.write(self.mgc)
        file.write(self.fmt)
        file.write(self.ver)
        file.write(b"\x00" * 3)
        file.write(self.mtime.to_bytes(8, "big"))
        file.write(b"\x00" * 8)
        file.write(self.localnode_id)
        file.write(b"\x00" * 4)
        file.write(len(self.nodes).to_bytes(4, "big"))
        file.write(b"\x00" * 4)
        for node in self.nodes:
            node.save(file)



# -*- coding: utf-8 -*-
from io import BytesIO
from os.path import dirname, join
from unittest import TestCase

from aioaria2 import ControlFile, DHTFile


class Testarser(TestCase):
    def test_ControlFile(self):
        s = BytesIO()
        data = ControlFile.from_file("180P_225K_242958531.webm.aria2")
        data.save(s)
        self.assertEqual(
            s.getvalue(), open("180P_225K_242958531.webm.aria2", "rb").read()
        )

    def test_DHTFile(self):
        s = BytesIO()
        data = DHTFile.from_file(join(dirname(__file__), "dht.dat"))
        data.save(s)
        self.assertEqual(
            len(s.getvalue()),
            len(open(join(dirname(__file__), "dht.dat"), "rb").read()),
        )

    def test_DHTFilev6(self):
        s = BytesIO()
        data = DHTFile.from_file(join(dirname(__file__), "dht6.dat"))
        data.save(s)
        self.assertEqual(
            len(s.getvalue()),
            len(open(join(dirname(__file__), "dht6.dat"), "rb").read()),
        )



# -*- coding: utf-8 -*-
import asyncio
import time
from pprint import pprint

import ujson

import aioaria2

HOST = "http://aria.blackjoe.art:2082/jsonrpc"


async def callback(trigger, data):
    print("下载开始{0}".format(data))


async def callback2(trigger: aioaria2.Aria2WebsocketTrigger, data):
    print("下载完成{0}".format(data))


async def callback3(trigger: aioaria2.Aria2WebsocketTrigger, future):
    print("下载中断")
    data = future.result()
    gid = data["params"][0]["gid"]
    trigger.identity = "getFiles"
    await trigger.getFiles(gid)
    trigger.identity = "id"


async def onresult(trigger, data):
    print("结果")
    if data["id"] == "getFiles":
        # 如同http一样的返回     getFiles调用
        url = data["result"][0]["uris"][0]["uri"]
        trigger.identity = "addUri"
        await trigger.addUri([url])
        trigger.identity = "id"
    elif data["id"] == "addUri":
        print("重新入队成功:{0}".format(data["result"]))


async def get_client():
    async with aioaria2.Aria2HttpClient(
        HOST, token="a489451594cda0792df1", loads=ujson.loads, dumps=ujson.dumps
    ) as client:
        # pprint(await client.addUri(["http://odrive.aptx.xin/%E5%8A%A8%E7%94%BB/2004/200445.zip"]))
        pprint(await client.getVersion())


async def get_trigger():
    try:
        client = await aioaria2.Aria2WebsocketTrigger.new(
            "https://127.0.0.1:443", token="a489451594cda0792df1"
        )
        # client=aioaria2.Aria2WebsocketTrigger("id", HOST,token="adman",)
        client.onDownloadStart(callback)
        client.onDownloadComplete(callback2)
        await client.addUri(["https://www.baidu.com"])
    except aioaria2.Aria2rpcException as e:
        print("can't connect ")


def main():
    loop = asyncio.get_event_loop()
    # loop.call_soon_threadsafe()
    # asyncio.run_coroutine_threadsafe()
    # tasks = [loop.create_task(get_trigger()),loop.create_task(get_client())]  # , loop.create_task(get_client())]
    tasks = [loop.create_task(get_trigger()), asyncio.sleep(100)]
    loop.run_until_complete(asyncio.wait(tasks))


if __name__ == "__main__":
    loop = asyncio.get_event_loop()
    loop.create_task(get_trigger())
    try:
        loop.run_forever()
    finally:
        loop.close()
    # main()
